@@ -139,6 +139,11 @@ using namespace TelEngine;
 #define MAX_STOP 5
 #endif
 
+// How many microseconds a worker sleeps on semaphore
+#ifndef WORKER_SLEEP
+#define WORKER_SLEEP 500000
+#endif
+
 // Supervisor control constants
 
 // Minimum configurable size of child's sanity pool
@@ -270,6 +275,7 @@ int Engine::s_haltcode = -1;
 int EnginePrivate::count = 0;
 static String s_cfgpath(CFG_PATH);
 static String s_usrpath;
+static String s_affinity;
 static bool s_createusr = true;
 static bool s_init = false;
 static bool s_dynplugin = false;
@@ -292,6 +298,7 @@ static String s_startMsg;
 static SharedVars s_vars;
 static Mutex s_hooksMutex(true,"HooksList");
 static ObjList s_hooks;
+static Semaphore* s_semWorkers = 0;
 static NamedCounter* s_counter = 0;
 static NamedCounter* s_workCnt = 0;
 
@@ -933,8 +940,17 @@ void EnginePrivate::run()
     setCurrentObjCounter(s_workCnt);
     for (;;) {
 	s_makeworker = false;
+	Semaphore* s = s_semWorkers;
+	if (s && Engine::self()->m_dispatcher.hasMessages())
+	    s->unlock();
 	Engine::self()->m_dispatcher.dequeue();
-	Thread::idle(true);
+	s = s_semWorkers;
+	if (s) {
+	    s->lock(WORKER_SLEEP);
+	    Thread::yield(true);
+	}
+	else
+	    Thread::idle(true);
     }
 }
 
@@ -1090,13 +1106,19 @@ static void superhandler(int signal)
 	::kill(s_childpid,signal);
 }
 
+#define SUPER_MSG(msg,...) \
+{ \
+    time_t t = ::time(0); \
+    ::fprintf(stderr,msg " on %s",##__VA_ARGS__,TelEngine::c_safe(::ctime(&t))); \
+}
+
 static void rotatelogs()
 {
     if (s_rotatenow) {
 	s_rotatenow = false;
-	::fprintf(stderr,"Supervisor (%d) closing the log file\n",s_superpid);
+	SUPER_MSG("Supervisor (%d) closing the log file",s_superpid);
 	logFileOpen();
-	::fprintf(stderr,"Supervisor (%d) reopening the log file\n",s_superpid);
+	SUPER_MSG("Supervisor (%d) reopening the log file",s_superpid);
     }
 }
 
@@ -1115,7 +1137,7 @@ static void copystream(int dest, int src)
 static int supervise(int initDelay)
 {
     s_superpid = ::getpid();
-    ::fprintf(stderr,"Supervisor (%d) is starting, max sanity %d\n",s_superpid,s_max_sanity);
+    SUPER_MSG("Supervisor (%d) is starting, max sanity %d",s_superpid,s_max_sanity);
     ::signal(SIGINT,superhandler);
     ::signal(SIGTERM,superhandler);
     ::signal(SIGHUP,superhandler);
@@ -1125,7 +1147,7 @@ static int supervise(int initDelay)
     ::signal(SIGUSR2,superhandler);
     ::signal(SIGALRM,SIG_IGN);
     if (initDelay > 0) {
-	::fprintf(stderr,"Supervisor (%d) delaying initial start by %u.%02u seconds\n",
+	SUPER_MSG("Supervisor (%d) delaying initial start by %u.%02u seconds",
 		s_superpid,(initDelay + 5) / 1000,((initDelay + 5) / 10) % 100);
 	::usleep(initDelay * 1000);
     }
@@ -1134,7 +1156,7 @@ static int supervise(int initDelay)
 	int wdogfd[2];
 	if (::pipe(wdogfd)) {
 	    int err = errno;
-	    ::fprintf(stderr,"Supervisor: watchdog pipe failed: %s (%d)\n",::strerror(err),err);
+	    SUPER_MSG("Supervisor: watchdog pipe failed: %s (%d)",::strerror(err),err);
 	    return err;
 	}
 	::fcntl(wdogfd[0],F_SETFL,O_NONBLOCK);
@@ -1143,7 +1165,7 @@ static int supervise(int initDelay)
 	if (s_logrotator) {
 	    if (::pipe(logfd)) {
 		int err = errno;
-		::fprintf(stderr,"Supervisor: log pipe failed: %s (%d)\n",::strerror(err),err);
+		SUPER_MSG("Supervisor: log pipe failed: %s (%d)",::strerror(err),err);
 		return err;
 	    }
 	    ::fcntl(logfd[0],F_SETFL,O_NONBLOCK);
@@ -1156,7 +1178,7 @@ static int supervise(int initDelay)
 	s_childpid = ::fork();
 	if (s_childpid < 0) {
 	    int err = errno;
-	    ::fprintf(stderr,"Supervisor: fork failed: %s (%d)\n",::strerror(err),err);
+	    SUPER_MSG("Supervisor: fork failed: %s (%d)",::strerror(err),err);
 	    return err;
 	}
 	if (s_childpid == 0) {
@@ -1189,7 +1211,7 @@ static int supervise(int initDelay)
 		// Child exited for some reason
 		if (WIFEXITED(status)) {
 		    retcode = WEXITSTATUS(status);
-		    ::fprintf(stderr,"Supervisor: child %d exited with code %d\n",s_childpid,retcode);
+		    SUPER_MSG("Supervisor: child %d exited with code %d",s_childpid,retcode);
 		    if (retcode <= 127)
 			s_runagain = false;
 		    else
@@ -1198,7 +1220,7 @@ static int supervise(int initDelay)
 		}
 		else if (WIFSIGNALED(status)) {
 		    retcode = WTERMSIG(status);
-		    ::fprintf(stderr,"Supervisor: child %d died on signal %d\n",s_childpid,retcode);
+		    SUPER_MSG("Supervisor: child %d died on signal %d",s_childpid,retcode);
 		    s_childsig = retcode;
 		}
 		s_childpid = -1;
@@ -1231,7 +1253,7 @@ static int supervise(int initDelay)
 	::close(wdogfd[0]);
 	if (s_childpid > 0) {
 	    // Child failed to proof sanity. Kill it - no need to be gentle.
-	    ::fprintf(stderr,"Supervisor: killing unresponsive child %d\n",s_childpid);
+	    SUPER_MSG("Supervisor: killing unresponsive child %d",s_childpid);
 #ifdef RLIMIT_CORE
 	    // If -Da or -C were specified try to get a corefile
 	    if (s_sigabrt || s_coredump) {
@@ -1260,7 +1282,7 @@ static int supervise(int initDelay)
 	}
 	if (s_runagain) {
 	    if (s_rundelay > RUNDELAY_MIN)
-		::fprintf(stderr,"Supervisor (%d) delaying child start by %u.%02u seconds\n",
+		SUPER_MSG("Supervisor (%d) delaying child start by %u.%02u seconds",
 		    s_superpid,s_rundelay / 1000000,(s_rundelay / 10000) % 100);
 	    ::usleep(s_rundelay);
 	    // Exponential backoff, double run delay each time we restart child
@@ -1269,9 +1291,12 @@ static int supervise(int initDelay)
 		s_rundelay = RUNDELAY_MAX;
 	}
     }
-    ::fprintf(stderr,"Supervisor (%d) exiting with code %d\n",s_superpid,retcode);
+    SUPER_MSG("Supervisor (%d) exiting with code %d",s_superpid,retcode);
     return retcode;
 }
+
+#undef SUPER_MSG
+
 #endif /* _WINDOWS */
 
 
@@ -1513,8 +1538,8 @@ int Engine::engineInit()
     const char *modPath = s_cfg.getValue("general","modpath");
     if (modPath)
 	s_modpath = modPath;
-    s_minworkers = s_cfg.getIntValue("general","minworkers",s_minworkers,1,100);
-    s_maxworkers = s_cfg.getIntValue("general","maxworkers",s_maxworkers,s_minworkers,500);
+    s_minworkers = s_cfg.getIntValue("general","minworkers",s_minworkers,1,500);
+    s_maxworkers = s_cfg.getIntValue("general","maxworkers",s_maxworkers,s_minworkers,1000);
     s_addworkers = s_cfg.getIntValue("general","addworkers",s_addworkers,1,10);
     s_maxmsgrate = s_cfg.getIntValue("general","maxmsgrate",s_maxmsgrate,0,50000);
     s_maxmsgage = s_cfg.getIntValue("general","maxmsgage",s_maxmsgage,0,5000);
@@ -1556,6 +1581,8 @@ int Engine::engineInit()
     s_params.addParam("maxmsgage",String(s_maxmsgage));
     s_params.addParam("maxqueued",String(s_maxqueued));
     s_params.addParam("maxevents",String(s_maxevents));
+    if (s_affinity)
+	s_params.addParam("affinity",s_affinity);
 
     int nodeBits = s_cfg.getIntValue("general","nodebits",0,0,10);
     if (nodeBits) {
@@ -1768,14 +1795,22 @@ int Engine::run()
 	    }
 	    else {
 		build = s_minworkers;
-		Debug(DebugInfo,"Creating first %d message dispatching threads",build);
+		if (!s_semWorkers && s_cfg.getBoolValue("general","semworkers",Semaphore::efficientTimedLock()))
+		    s_semWorkers = new Semaphore(build,"Workers",0);
+		Debug(DebugInfo,"Creating first %d message dispatching threads%s",build,
+		    (s_semWorkers ? " and semaphore" : ""));
 	    }
 	    do {
 		(new EnginePrivate)->startup();
 	    } while (--build > 0);
 	}
-	else
+	else {
 	    s_makeworker = true;
+	    // Wake at least one worker into action
+	    Semaphore* s = s_semWorkers;
+	    if (s)
+		s->unlock();
+	}
 
 	uint64_t now = Time::now();
 	if (last) {
@@ -1865,6 +1900,12 @@ int Engine::engineCleanup()
     myLock.drop();
     dispatch("engine.halt",true);
     checkPoint();
+    Semaphore* s = s_semWorkers;
+    s_semWorkers = 0;
+    if (s) {
+	for (int i = EnginePrivate::count; i > 0; i--)
+	    s->unlock();
+    }
     Thread::msleep(200);
     m_dispatcher.dequeue();
     checkPoint();
@@ -1886,6 +1927,7 @@ int Engine::engineCleanup()
     plugins.clear();
     if (mux || cnt)
 	Debug(DebugWarn,"Exiting with %d locked mutexes and %u plugins loaded!",mux,cnt);
+    delete s;
     if (GenObject::getObjCounting()) {
 	String str;
 	int obj = EngineStatusHandler::objects(str);
@@ -2243,7 +2285,13 @@ bool Engine::enqueue(Message* msg, bool skipHooks)
 	    return true;
 	}
     }
-    return s_self ? s_self->m_dispatcher.enqueue(msg) : false;
+    if (s_self && s_self->m_dispatcher.enqueue(msg)) {
+	Semaphore*s = s_semWorkers;
+	if (s)
+	    s->unlock();
+	return true;
+    }
+    return false;
 }
 
 bool Engine::dispatch(Message* msg)
@@ -2491,6 +2539,7 @@ static void usage(bool client, FILE* f)
 "   -x dirpath     Absolute or relative path to extra modules directory (can be repeated)\n"
 "   -w directory   Change working directory\n"
 "   -N nodename    Set the name of this node in a cluster\n"
+"   -A cpus        Set affinity from comma separated list of CPUs (e.g 1-4,7,8)\n"
 #ifdef RLIMIT_CORE
 "   -C             Enable core dumps if possible\n"
 #endif
@@ -2721,6 +2770,10 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 			GET_PARAM;
 			s_node = param;
 			break;
+		    case 'A':
+			GET_PARAM;
+			s_affinity = param;
+			break;
 #ifdef RLIMIT_CORE
 		    case 'C':
 			s_coredump = true;
@@ -2985,6 +3038,13 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
     }
 #endif
 
+    if (s_affinity) {
+	int err = Thread::setCurrentAffinity(s_affinity);
+	if (err) {
+	    Debug(DebugWarn,"Failed to set affinity to '%s', error=%s(%d)",s_affinity.c_str(),strerror(err),err);
+	    s_affinity.clear();
+	}
+    }
     int retcode = -1;
 #ifndef _WINDOWS
     if (supervised)
