@@ -3,7 +3,7 @@
  * This file is part of the YATE Project http://YATE.null.ro
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2013 Null Team
+ * Copyright (C) 2004-2014 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -210,6 +210,31 @@ ObjList* ObjList::append(const GenObject* obj, bool compact)
     return n;
 }
 
+ObjList* ObjList::setUnique(const GenObject* obj, bool compact)
+{
+    XDebug(DebugAll,"ObjList::setUnique(\"%p\") [%p]",obj,this);
+    if (!obj)
+	return 0;
+    const String& name = obj->toString();
+    ObjList* o = skipNull();
+    while (o) {
+	if (name.matches(o->get()->toString())) {
+	    o->set(obj);
+	    return const_cast<ObjList*>(o);
+	}
+	ObjList* n = o->skipNext();
+	if (n)
+	    o = n;
+	else
+	    break;
+    }
+    if (o)
+	o = o->append(obj,compact);
+    else
+	o = append(obj,compact);
+    return const_cast<ObjList*>(o);
+}
+
 GenObject* ObjList::remove(bool delobj)
 {
     GenObject *tmp = m_obj;
@@ -325,14 +350,18 @@ static void merge(ObjList* first, ObjList* second,int (*compare)(GenObject* obj1
 {
     if (!(first && second))
 	return;
+    bool del = true;
     if (!first->skipNull()) {
-	while (second->skipNull())
-	    first->append(second->remove(false));
+	while (second->skipNull()) {
+	    del = second->autoDelete();
+	    first->append(second->remove(false))->setDelete(del);
+	}
 	return;
     }
     ObjList* head = first->skipNull();
     GenObject* current = head->get();
     while (second->skipNull()) {
+	del = second->autoDelete();
 	GenObject* next = second->remove(false);
 	while (current && compare(current,next,dat) < 1) {
 	    if (!head->skipNext()) {
@@ -343,10 +372,10 @@ static void merge(ObjList* first, ObjList* second,int (*compare)(GenObject* obj1
 	    current = head->get();
 	}
 	if (!current) {
-	    first->append(next);
+	    first->append(next)->setDelete(del);
 	    continue;
 	}
-	head->insert(next);
+	head->insert(next)->setDelete(del);
 	head = head->skipNext();
     }
 }
@@ -357,17 +386,19 @@ static void splitList(ObjList& list, ObjList& splits, int (*compare)(GenObject*
 	return;
     ObjList* slice = new ObjList();
     splits.append(slice);
+    bool autodel = list.autoDelete();
     GenObject* last = list.remove(false);
-    slice->append(last);
+    slice->append(last)->setDelete(autodel);
     while (list.skipNull()) {
+	autodel = list.autoDelete();
 	GenObject* next = list.remove(false);
 	if (compare(last,next,data) < 1) {
-	    slice->append(next);
+	    slice->append(next)->setDelete(autodel);
 	    last = next;
 	    continue;
 	}
 	slice = new ObjList();
-	slice->append(next);
+	slice->append(next)->setDelete(autodel);
 	splits.append(slice);
 	last = next;
     }
