@@ -252,8 +252,9 @@ static void MD5_Final(unsigned char digest[16], MD5_CTX *ctx)
     byteReverse(ctx->in, 14);
 
     /* Append length in bits and transform */
-    ((u_int32_t *) ctx->in)[14] = ctx->bits[0];
-    ((u_int32_t *) ctx->in)[15] = ctx->bits[1];
+    uint32_t* ctxIn = (uint32_t*)ctx->in;
+    ctxIn[14] = ctx->bits[0];
+    ctxIn[15] = ctx->bits[1];
 
     MD5Transform(ctx->buf, (u_int32_t *) ctx->in);
     byteReverse((unsigned char *) ctx->buf, 4);
