@@ -24,6 +24,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <math.h>
 
 using namespace TelEngine;
 
@@ -373,11 +374,36 @@ bool ExpEvaluator::getNumber(ParsePoint& expr)
     if (inError())
 	return false;
     XDebug(this,DebugAll,"getNumber line=0x%X '%.30s'",lineNumber(),(const char*)expr);
+    static const Regexp r("^[-+]?([0-9]*(\\.)?[0-9]+|[0-9]+(\\.))([eE][-+]?[0-9]+)?",true);
+    String m(expr,50);
+    if (m.matches(r) && (m.matchLength(2) || m.matchLength(3) || m.matchLength(4))) {
+	String str = m.matchString();
+	double val = str.toDouble(0.0);
+	// Maximum IEEE double that can be converted to signed int64
+#define DLONG_MAX 9223372036854774784.0
+	if (val >= DLONG_MAX)
+	    val = DLONG_MAX;
+	else if (val <= -DLONG_MAX)
+	    val = -DLONG_MAX;
+#undef DLONG_MAX
+	expr += str.length();
+	ExpOperation* op = addOpcode(str);
+#ifdef _WINDOWS
+	op->m_number = (int64_t)((val >= 0) ? (val + 0.5) : (val - 0.5));
+#else
+	op->m_number = ::round(val);
+#endif
+	op->m_isNumber = true;
+	DDebug(this,DebugAll,"Fake float %s ~ " FMT64,str.safe(),op->m_number);
+	return true;
+    }
     char* endp = 0;
     int64_t val = ::strtoll(expr,&endp,0);
     if (!endp || (endp == expr))
 	return false;
     expr = endp;
+    if (LLONG_MIN == val)
+	val++;
     DDebug(this,DebugAll,"Found " FMT64,val);
     addOpcode(val);
     return true;
@@ -393,6 +419,7 @@ bool ExpEvaluator::getString(ParsePoint& expr)
 	String str;
 	if (getString(expr,str)) {
 	    addOpcode(str);
+	    expr.m_lineNo = m_lineNo;
 	    return true;
 	}
     }
@@ -403,6 +430,7 @@ bool ExpEvaluator::getString(const char*& expr, String& str)
 {
     char sep = *expr++;
     const char* start = expr;
+    unsigned int startLine = m_lineNo;
     while (char c = *expr++) {
 	if (c != '\\' && c != sep)
 	    continue;
@@ -419,12 +447,15 @@ bool ExpEvaluator::getString(const char*& expr, String& str)
 	start = expr;
     }
     expr--;
+    m_lineNo = startLine;
     return gotError("Expecting string end");
 }
 
 bool ExpEvaluator::getEscape(const char*& expr, String& str, char sep)
 {
     char c = *expr++;
+    if (c == '\n')
+	++m_lineNo;
     switch (c) {
 	case '\0':
 	    return false;
@@ -994,8 +1025,10 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 	    break;
 	case OpcCopy:
 	    {
-		Mutex* mtx = 0;
+		ScriptMutex* mtx = 0;
 		ScriptRun* runner = YOBJECT(ScriptRun,&oper);
+		if (!runner)
+		    runner = YOBJECT(ScriptRun,context);
 		if (runner) {
 		    if (runner->context())
 			mtx = runner->context()->mutex();
@@ -1345,8 +1378,8 @@ bool ExpEvaluator::runOperation(ObjList& stack, const ExpOperation& oper, GenObj
 
 bool ExpEvaluator::runFunction(ObjList& stack, const ExpOperation& oper, GenObject* context) const
 {
-    DDebug(this,DebugAll,"runFunction(%p,'%s' " FMT64 ", %p) ext=%p",
-	&stack,oper.name().c_str(),oper.number(),context,(void*)m_extender);
+    DDebug(this,DebugAll,"runFunction(%p,'%s' " FMT64 ", %p) ext=%p line=0x%08x",
+	&stack,oper.name().c_str(),oper.number(),context,(void*)m_extender,oper.lineNumber());
     if (oper.name() == YSTRING("chr")) {
 	String res;
 	for (int i = (int)oper.number(); i; i--) {
@@ -1605,13 +1638,13 @@ ExpOperation* ExpWrapper::clone(const char* name) const
     return op;
 }
 
-ExpOperation* ExpWrapper::copy(Mutex* mtx) const
+ExpOperation* ExpWrapper::copy(ScriptMutex* mtx) const
 {
     JsObject* jso = YOBJECT(JsObject,m_object);
     if (!jso)
 	return ExpOperation::clone();
     XDebug(DebugInfo,"ExpWrapper::copy(%p) [%p]",mtx,this);
-    ExpWrapper* op = new ExpWrapper(jso->copy(mtx),name());
+    ExpWrapper* op = new ExpWrapper(jso->copy(mtx,*this),name());
     static_cast<String&>(*op) = *this;
     op->lineNumber(lineNumber());
     return op;
