@@ -30,7 +30,7 @@ class JsObjectObj : public JsObject
 {
     YCLASS(JsObjectObj,JsObject)
 public:
-    inline JsObjectObj(Mutex* mtx)
+    inline JsObjectObj(ScriptMutex* mtx)
 	: JsObject("Object",mtx,true)
 	{
 	}
@@ -47,7 +47,7 @@ class JsDate : public JsObject
 {
     YCLASS(JsDate,JsObject)
 public:
-    inline JsDate(Mutex* mtx)
+    inline JsDate(ScriptMutex* mtx)
 	: JsObject("Date",mtx,true),
 	  m_time(0), m_msec(0), m_offs(0)
 	{
@@ -86,16 +86,16 @@ public:
 	}
 
 protected:
-    inline JsDate(Mutex* mtx, u_int64_t msecs, bool local = false)
-	: JsObject("Date",mtx),
-	  m_time((unsigned int)(msecs / 1000)), m_msec((unsigned int)(msecs % 1000)), m_offs(Time::timeZone())
+    inline JsDate(ScriptMutex* mtx, unsigned int lineNo, u_int64_t msecs, bool local = false)
+	: JsObject(mtx,"[object Date]",lineNo),
+	  m_time((unsigned int)(msecs / 1000)), m_msec((unsigned int)(msecs % 1000)), m_offs(Time::timeZone(m_time))
 	{ if (local) m_time -= m_offs; }
-    inline JsDate(Mutex* mtx, const char* name, unsigned int time, unsigned int msec, unsigned int offs)
-	: JsObject(mtx,name),
+    inline JsDate(ScriptMutex* mtx, const char* name, unsigned int line, unsigned int time, unsigned int msec, unsigned int offs)
+	: JsObject(mtx,name,line),
 	  m_time(time), m_msec(msec), m_offs(offs)
 	{ }
-    virtual JsObject* clone(const char* name) const
-	{ return new JsDate(mutex(),name,m_time,m_msec,m_offs); }
+    virtual JsObject* clone(const char* name, const ExpOperation& oper) const
+	{ return new JsDate(mutex(),name,oper.lineNumber(),m_time,m_msec,m_offs); }
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
 private:
     unsigned int m_time;
@@ -109,7 +109,7 @@ class JsMath : public JsObject
 {
     YCLASS(JsMath,JsObject)
 public:
-    inline JsMath(Mutex* mtx)
+    inline JsMath(ScriptMutex* mtx)
 	: JsObject("Math",mtx,true)
 	{
 	    params().addParam(new ExpFunction("abs"));
@@ -125,10 +125,17 @@ protected:
 
 
 // Helper function that does the actual object printing
-static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int depth, ObjList& seen)
+static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int depth, ObjList& seen,
+    unsigned int flags)
 {
     if (!obj)
 	return;
+    if (depth > 1 && !(flags & JsObject::DumpRecursive))
+	return;
+    // Check if we have something to dump
+    unsigned int dump = flags & (JsObject::DumpFunc | JsObject::DumpProp);
+    if (!dump)
+	return;
     String str(' ',2 * depth);
     if (seen.find(obj)) {
 	str << "(recursivity encountered)";
@@ -136,12 +143,17 @@ static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int dep
 	return;
     }
     const NamedString* nstr = YOBJECT(NamedString,obj);
+    // Check for prototype dump (always dump the first level if original object is a prototype)
+    bool isProto = nstr && nstr->name() == JsObject::protoName();
+    if (depth && isProto && !(flags & JsObject::DumpProto))
+	return;
     const NamedPointer* nptr = YOBJECT(NamedPointer,nstr);
     const char* type = nstr ? (nptr ? "NamedPointer" : "NamedString") : "???";
     const char* subType = 0;
     const ScriptContext* scr = YOBJECT(ScriptContext,obj);
     const ExpWrapper* wrap = 0;
     bool objRecursed = false;
+    bool isFunc = false;
     if (scr) {
 	const JsObject* jso = YOBJECT(JsObject,scr);
 	if (jso) {
@@ -150,8 +162,10 @@ static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int dep
 		seen.append(jso)->setDelete(false);
 	    if (YOBJECT(JsArray,scr))
 		type = "JsArray";
-	    else if (YOBJECT(JsFunction,scr))
+	    else if (YOBJECT(JsFunction,scr)) {
 		type = "JsFunction";
+		isFunc = true;
+	    }
 	    else if (YOBJECT(JsRegExp,scr))
 		type = "JsRegExp";
 	    else if (YOBJECT(JsDate,scr))
@@ -167,46 +181,98 @@ static void dumpRecursiveObj(const GenObject* obj, String& buf, unsigned int dep
     if (exp && !scr) {
 	if ((wrap = YOBJECT(ExpWrapper,exp)))
 	    type = wrap->object() ? "ExpWrapper" : "Undefined";
-	else if (YOBJECT(ExpFunction,exp))
+	else if (YOBJECT(ExpFunction,exp)) {
 	    type = "ExpFunction";
+	    isFunc = true;
+	}
 	else {
 	    type = "ExpOperation";
 	    subType = exp->typeOf();
 	}
     }
-    if (nstr)
-	str << "'" << nstr->name() << "' = '" << *nstr << "'";
+    // Check for func/prop dump (don't do it if we are printing a prototype)
+    if (depth && !isProto && ((isFunc && (0 == (flags & JsObject::DumpFunc))) ||
+	(!isFunc && (0 == (flags & JsObject::DumpProp)))))
+	return;
+    bool dumpType = 0 != (flags & JsObject::DumpType);
+    if (nstr) {
+	str << "'" << nstr->name() << "'";
+	// Nicely dump property value if dumping props only and type is not shown 
+	if ((dump == JsObject::DumpProp) && !isProto && !dumpType) {
+	    if (scr) {
+		if (exp && JsParser::isNull(*exp))
+		    str << " = null";
+		else if (YOBJECT(JsRegExp,scr))
+		    str << " = /" << *nstr << "/";
+		else if (flags & JsObject::DumpPropObjType) {
+		    if (YOBJECT(JsObject,scr))
+			str << " = " << *nstr;
+		    else
+			str << " = [ScriptContext]";
+		}
+	    }
+	    else if (exp) {
+		if (JsParser::isUndefined(*exp))
+		    str << " = undefined";
+		else if (exp->isInteger()) {
+		    if (exp->isBoolean())
+			str << " = " << exp->valBoolean();
+		    else
+			str << " = " << exp->number();
+		}
+		else if (exp->isNumber()) // NaN
+		    str << " = " << *nstr;
+		else // string
+		    str << " = '" << *nstr << "'";
+	    }
+	    else
+		str << " = '" << *nstr << "'";
+	}
+	else
+	    str << " = '" << *nstr << "'";
+    }
     else
 	str << "'" << obj->toString() << "'";
-    str << " (" << type << (subType ? ", " : "") << subType << ")";
+    if (dumpType)
+	str << " (" << type << (subType ? ", " : "") << subType << ")";
     if (objRecursed)
 	str << " (already seen)";
     buf.append(str,"\r\n");
     if (objRecursed)
 	return;
-    str.clear();
     if (scr) {
 	NamedIterator iter(scr->params());
 	while (const NamedString* p = iter.get())
-	    dumpRecursiveObj(p,buf,depth + 1,seen);
+	    dumpRecursiveObj(p,buf,depth + 1,seen,flags);
 	if (scr->nativeParams()) {
 	    iter = *scr->nativeParams();
 	    while (const NamedString* p = iter.get())
-		dumpRecursiveObj(p,buf,depth + 1,seen);
+		dumpRecursiveObj(p,buf,depth + 1,seen,flags);
 	}
     }
     else if (wrap)
-	dumpRecursiveObj(wrap->object(),buf,depth + 1,seen);
+	dumpRecursiveObj(wrap->object(),buf,depth + 1,seen,flags);
     else if (nptr)
-	dumpRecursiveObj(nptr->userData(),buf,depth + 1,seen);
+	dumpRecursiveObj(nptr->userData(),buf,depth + 1,seen,flags);
+    const JsObject* jso = YOBJECT(JsObject,obj);
+    if (jso) {
+	const HashList* hash = jso->getHashListParams();
+	if (hash) {
+	    for (unsigned int i = 0; i < hash->length(); i++) {
+		ObjList* lst = hash->getList(i);
+		for (lst ? lst = lst->skipNull() : 0; lst; lst = lst->skipNext())
+		    dumpRecursiveObj(lst->get(),buf,depth + 1,seen,flags);
+	    }
+	}
+    }
 }
 
 
 const String JsObject::s_protoName("__proto__");
 
-JsObject::JsObject(const char* name, Mutex* mtx, bool frozen)
+JsObject::JsObject(const char* name, ScriptMutex* mtx, bool frozen)
     : ScriptContext(String("[object ") + name + "]"),
-      m_frozen(frozen), m_mutex(mtx)
+      m_frozen(frozen), m_mutex(mtx), m_lineNo(0)
 {
     XDebug(DebugAll,"JsObject::JsObject('%s',%p,%s) [%p]",
 	name,mtx,String::boolText(frozen),this);
@@ -216,43 +282,51 @@ JsObject::JsObject(const char* name, Mutex* mtx, bool frozen)
     params().addParam(new ExpFunction("hasOwnProperty"));
 }
 
-JsObject::JsObject(Mutex* mtx, const char* name, bool frozen)
+JsObject::JsObject(ScriptMutex* mtx, const char* name, unsigned int line, bool frozen)
     : ScriptContext(name),
-      m_frozen(frozen), m_mutex(mtx)
+      m_frozen(frozen), m_mutex(mtx), m_lineNo(line)
 {
-    XDebug(DebugAll,"JsObject::JsObject(%p,'%s',%s) [%p]",
-	mtx,name,String::boolText(frozen),this);
+    XDebug(DebugAll,"JsObject::JsObject(%p,'%s',0x%08x,%s) [%p]",
+	mtx,name,m_lineNo,String::boolText(frozen),this);
+    if (/*m_lineNo && */ m_mutex && m_mutex->objTrack())
+	m_mutex->objCreated(this);
 }
 
-JsObject::JsObject(GenObject* context, Mutex* mtx, bool frozen)
+JsObject::JsObject(GenObject* context, unsigned int line, ScriptMutex* mtx, bool frozen)
     : ScriptContext("[object Object]"),
-      m_frozen(frozen), m_mutex(mtx)
+      m_frozen(frozen), m_mutex(mtx), m_lineNo(line)
 {
+    XDebug(DebugAll,"JsObject::JsObject(ctxt=%p,l=0x%08x,mtx=%p,f=%s) [%p]",
+	context,m_lineNo,mtx,String::boolText(frozen),this);
     setPrototype(context,YSTRING("Object"));
+    if (/*m_lineNo && */ m_mutex && m_mutex->objTrack())
+	m_mutex->objCreated(this);
 }
 
 JsObject::~JsObject()
 {
+    if (m_mutex && m_mutex->objTrack())
+	m_mutex->objDeleted(this);
     XDebug(DebugAll,"JsObject::~JsObject '%s' [%p]",toString().c_str(),this);
 }
 
-JsObject* JsObject::copy(Mutex* mtx) const
+JsObject* JsObject::copy(ScriptMutex* mtx, const ExpOperation& oper) const
 {
-    JsObject* jso = new JsObject(mtx,toString(),frozen());
+    JsObject* jso = new JsObject(mtx,toString(),oper.lineNumber(),frozen());
     deepCopyParams(jso->params(),params(),mtx);
     return jso;
 }
 
-void JsObject::dumpRecursive(const GenObject* obj, String& buf)
+void JsObject::dumpRecursive(const GenObject* obj, String& buf, unsigned int flags)
 {
     ObjList seen;
-    dumpRecursiveObj(obj,buf,0,seen);
+    dumpRecursiveObj(obj,buf,0,seen,flags);
 }
 
-void JsObject::printRecursive(const GenObject* obj)
+void JsObject::printRecursive(const GenObject* obj, unsigned int flags)
 {
     String buf;
-    dumpRecursive(obj,buf);
+    dumpRecursive(obj,buf,flags);
     Output("%s",buf.c_str());
 }
 
@@ -304,12 +378,61 @@ ExpOperation* JsObject::toJSON(const ExpOperation* oper, int spaces)
     return ret;
 }
 
-void JsObject::toJSON(const NamedString* ns, String& buf, int spaces, int indent)
+// Utility: retrieve a JSON candidate from given list item
+// Advance the list
+// Return pointer to candidate, NULL if not found
+static inline GenObject* nextJSONCandidate(ObjList*& crt, bool isNs = true)
 {
-    const ExpOperation* oper = YOBJECT(ExpOperation,ns);
+    if (!crt)
+	return 0;
+    if (!crt->get()) {
+	crt = crt->skipNull();
+	if (!crt)
+	    return 0;
+    }
+    while (crt) {
+	GenObject* gen = crt->get();
+	crt = crt->skipNext();
+	const String& n = isNs ? static_cast<NamedString*>(gen)->name() : gen->toString();
+	if (!n || n == JsObject::protoName() || YOBJECT(JsFunction,gen) || YOBJECT(ExpFunction,gen))
+	    continue;
+	const ExpOperation* op = YOBJECT(ExpOperation,gen);
+	if (!(op && JsParser::isUndefined(*op)))
+	    return gen;
+    }
+    return 0;
+}
+
+// Utility: retrieve a JSON candidate from given hash list
+// Advance the list
+// Return pointer to candidate, NULL if not found
+static inline GenObject* nextJSONCandidate(const HashList& hash, unsigned int& idx, ObjList*& crt)
+{
+    GenObject* gen = nextJSONCandidate(crt,false);
+    if (gen)
+	return gen;
+    crt = 0;
+    while (++idx < hash.length()) {
+	crt = hash.getList(idx);
+	if (!crt)
+	    continue;
+	gen = nextJSONCandidate(crt,false);
+	if (gen)
+	    return gen;
+    }
+    return 0;
+}
+
+void JsObject::internalToJSON(const GenObject* obj, bool isStr, String& buf, int spaces, int indent)
+{
+    if (!obj) {
+	buf << "null";
+	return;
+    }
+    const ExpOperation* oper = YOBJECT(ExpOperation,obj);
     if (!oper) {
-	if (ns)
-	    buf << strEscape(*ns);
+	if (isStr)
+	    buf << strEscape(*static_cast<const String*>(obj));
 	else
 	    buf << "null";
 	return;
@@ -352,6 +475,30 @@ void JsObject::toJSON(const NamedString* ns, String& buf, int spaces, int indent
 	    buf << strEscape(jso->toString());
 	    return;
 	}
+	const HashList* hash = jso->getHashListParams();
+	if (hash) {
+	    ObjList* crt = hash->getList(0);
+	    unsigned int idx = 0;
+	    GenObject* gen = nextJSONCandidate(*hash,idx,crt);
+	    if (!gen) {
+		buf << "{}";
+		return;
+	    }
+	    String li(' ',indent);
+	    String ci(' ',indent + spaces);
+	    const char* sep = spaces ? ": " : ":";
+	    buf << "{" << nl;
+	    while (gen) {
+		buf << ci << strEscape(gen->toString()) << sep;
+		internalToJSON(gen,false,buf,spaces,indent + spaces);
+		gen = static_cast<NamedString*>(nextJSONCandidate(*hash,idx,crt));
+		if (gen)
+		    buf << ",";
+		buf << nl;
+	    }
+	    buf << li << "}";
+	    return;
+	}
 	switch (jso->params().count()) {
 	    case 1:
 		if (!jso->params().getParam(protoName()))
@@ -366,24 +513,12 @@ void JsObject::toJSON(const NamedString* ns, String& buf, int spaces, int indent
 	String ci(' ',indent + spaces);
 	const char* sep = spaces ? ": " : ":";
 	buf << "{" << nl;
-	while (l) {
-	    const NamedString* p = static_cast<const NamedString*>(l->get());
-	    l = l->skipNext();
-	    if (p->name() == protoName() || YOBJECT(JsFunction,p) || YOBJECT(ExpFunction,p))
-		continue;
-	    const ExpOperation* op = YOBJECT(ExpOperation,p);
-	    if (op && JsParser::isUndefined(*op))
-		continue;
+	const NamedString* p = static_cast<NamedString*>(nextJSONCandidate(l));
+	while (p) {
 	    buf << ci << strEscape(p->name()) << sep;
-	    toJSON(p,buf,spaces,indent + spaces);
-	    for (; l; l = l->skipNext()) {
-		p = static_cast<const NamedString*>(l->get());
-		op = YOBJECT(ExpOperation,p);
-		if (!(p->name() == protoName() || YOBJECT(JsFunction,p) || YOBJECT(ExpFunction,p)
-			|| (op && JsParser::isUndefined(*op))))
-		    break;
-	    }
-	    if (l)
+	    internalToJSON(p,true,buf,spaces,indent + spaces);
+	    p = static_cast<NamedString*>(nextJSONCandidate(l));
+	    if (p)
 		buf << ",";
 	    buf << nl;
 	}
@@ -418,11 +553,13 @@ void JsObject::setPrototype(GenObject* context, const String& objName)
     }
 }
 
-JsObject* JsObject::buildCallContext(Mutex* mtx, JsObject* thisObj)
+JsObject* JsObject::buildCallContext(ScriptMutex* mtx, JsObject* thisObj)
 {
-    JsObject* ctxt = new JsObject(mtx,"()");
-    if (thisObj && thisObj->alive())
+    JsObject* ctxt = new JsObject(mtx,"()",0);
+    if (thisObj && thisObj->alive()) {
+	ctxt->lineNo(thisObj->lineNo());
 	ctxt->params().addParam(new ExpWrapper(thisObj,"this"));
+    }
     return ctxt;
 }
 
@@ -439,6 +576,11 @@ void JsObject::fillFieldNames(ObjList& names)
 #endif
 }
 
+const HashList* JsObject::getHashListParams() const
+{
+    return 0;
+}
+
 bool JsObject::hasField(ObjList& stack, const String& name, GenObject* context) const
 {
     if (ScriptContext::hasField(stack,name,context))
@@ -471,7 +613,7 @@ JsObject* JsObject::runConstructor(ObjList& stack, const ExpOperation& oper, Gen
 {
     if (!ref())
 	return 0;
-    JsObject* obj = clone("[object " + oper.name() + "]");
+    JsObject* obj = clone("[object " + oper.name() + "]",oper);
     obj->params().addParam(new ExpWrapper(this,protoName()));
     return obj;
 }
@@ -597,7 +739,7 @@ void JsObject::addObject(NamedList& params, const char* name, JsObject* obj)
 // Static method that adds a constructor to a parent
 void JsObject::addConstructor(NamedList& params, const char* name, JsObject* obj)
 {
-    JsFunction* ctr = new JsFunction(obj->mutex(),name);
+    JsFunction* ctr = new JsFunction(obj->mutex(),name,0);
     ctr->params().addParam(new NamedPointer("prototype",obj,obj->toString()));
     obj->initConstructor(ctr);
     params.addParam(new NamedPointer(name,ctr,ctr->toString()));
@@ -620,7 +762,7 @@ int JsObject::extractArgs(JsObject* obj, ObjList& stack, const ExpOperation& ope
 }
 
 // Static helper method that deep copies all parameters
-void JsObject::deepCopyParams(NamedList& dst, const NamedList& src, Mutex* mtx)
+void JsObject::deepCopyParams(NamedList& dst, const NamedList& src, ScriptMutex* mtx)
 {
     NamedIterator iter(src);
     while (const NamedString* p = iter.get()) {
@@ -637,7 +779,7 @@ void JsObject::initialize(ScriptContext* context)
 {
     if (!context)
 	return;
-    Mutex* mtx = context->mutex();
+    ScriptMutex* mtx = context->mutex();
     Lock mylock(mtx);
     NamedList& p = context->params();
     static_cast<String&>(p) = "[object Global]";
@@ -655,6 +797,24 @@ void JsObject::initialize(ScriptContext* context)
 	addObject(p,"Math",new JsMath(mtx));
 }
 
+void JsObject::setLineForObj(JsObject* obj,unsigned int lineNo, bool recursive)
+{
+    if (!obj)
+	return;
+    DDebug(DebugAll,"JsObject::setLineForObj(%p,%u,%s)",obj,lineNo,String::boolText(recursive));
+    obj->lineNo(lineNo);
+    if (!recursive)
+	return;
+    for (unsigned int i = 0;i < obj->params().length();i++) {
+	String* param = obj->params().getParam(i);
+	JsObject* tmpObj = YOBJECT(JsObject,param);
+	if (!tmpObj)
+	    continue;
+	JsObject::setLineForObj(tmpObj,lineNo,recursive);
+	tmpObj->lineNo(lineNo);
+    }
+}
+
 bool JsObject::getIntField(const String& name, int64_t& val)
 {
     NamedString* ns = params().getParam(name);
@@ -722,7 +882,7 @@ bool JsObjectObj::runNative(ObjList& stack, const ExpOperation& oper, GenObject*
 	const NamedList* lst = YOBJECT(NamedList,obj);
 	if (lst) {
 	    NamedIterator iter(*lst);
-	    JsArray* jsa = new JsArray(context,mutex());
+	    JsArray* jsa = new JsArray(context,oper.lineNumber(),mutex());
 	    while (const NamedString* ns = iter.get())
 		if (ns->name() != protoName())
 		    jsa->push(new ExpOperation(ns->name(),0,true));
@@ -738,7 +898,7 @@ bool JsObjectObj::runNative(ObjList& stack, const ExpOperation& oper, GenObject*
 }
 
 
-JsArray::JsArray(Mutex* mtx)
+JsArray::JsArray(ScriptMutex* mtx)
     : JsObject("Array",mtx), m_length(0)
 {
     params().addParam(new ExpFunction("push"));
@@ -757,15 +917,15 @@ JsArray::JsArray(Mutex* mtx)
     params().addParam("length","0");
 }
 
-JsArray::JsArray(GenObject* context, Mutex* mtx)
-    : JsObject(mtx,"[object Array]"), m_length(0)
+JsArray::JsArray(GenObject* context, unsigned int line, ScriptMutex* mtx)
+    : JsObject(mtx,"[object Array]",line), m_length(0)
 {
     setPrototype(context,YSTRING("Array"));
 }
 
-JsObject* JsArray::copy(Mutex* mtx) const
+JsObject* JsArray::copy(ScriptMutex* mtx, const ExpOperation& oper) const
 {
-    JsArray* jsa = new JsArray(mtx,toString(),frozen());
+    JsArray* jsa = new JsArray(mtx,toString(),oper.lineNumber(),frozen());
     deepCopyParams(jsa->params(),params(),mtx);
     jsa->setLength(length());
     return jsa;
@@ -825,7 +985,7 @@ JsObject* JsArray::runConstructor(ObjList& stack, const ExpOperation& oper, GenO
 {
     if (!ref())
 	return 0;
-    JsArray* obj = static_cast<JsArray*>(clone("[object " + oper.name() + "]"));
+    JsArray* obj = static_cast<JsArray*>(clone("[object " + oper.name() + "]",oper));
     unsigned int len = (unsigned int)oper.number();
     for (unsigned int i = len; i;  i--) {
 	ExpOperation* op = obj->popValue(stack,context);
@@ -898,7 +1058,7 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	ObjList args;
 	extractArgs(this,stack,oper,context,args);
 
-	JsArray* array = new JsArray(context,mutex());
+	JsArray* array = new JsArray(context,oper.lineNumber(),mutex());
 	// copy this array - only numerically indexed elements!
 	for (int i = 0; i < m_length; i++) {
 	    NamedString* ns = params().getParam(String(i));
@@ -1167,8 +1327,8 @@ bool JsArray::runNativeSlice(ObjList& stack, const ExpOperation& oper, GenObject
     }
     if (end < 0)
 	end = length() + end;
- 
-    JsArray* array = new JsArray(context,mutex());
+
+    JsArray* array = new JsArray(context,oper.lineNumber(),mutex());
     for (int32_t i = begin; i < end; i++) {
 	NamedString* ns = params().getParam(String(i));
 	if (!ns) {
@@ -1218,7 +1378,7 @@ bool JsArray::runNativeSplice(ObjList& stack, const ExpOperation& oper, GenObjec
     }
 
     // remove elements
-    JsArray* removed = new JsArray(context,mutex());
+    JsArray* removed = new JsArray(context,oper.lineNumber(),mutex());
     for (int32_t i = begin; i < begin + delCount; i++) {
 	NamedString* ns = params().getParam(String(i));
 	if (!ns) {
@@ -1346,15 +1506,15 @@ bool JsArray::runNativeSort(ObjList& stack, const ExpOperation& oper, GenObject*
 }
 
 
-JsRegExp::JsRegExp(Mutex* mtx)
+JsRegExp::JsRegExp(ScriptMutex* mtx)
     : JsObject("RegExp",mtx)
 {
     params().addParam(new ExpFunction("test"));
     params().addParam(new ExpFunction("valid"));
 }
 
-JsRegExp::JsRegExp(Mutex* mtx, const char* name, const char* rexp, bool insensitive, bool extended, bool frozen)
-    : JsObject(mtx,name,frozen),
+JsRegExp::JsRegExp(ScriptMutex* mtx, const char* name, unsigned int line, const char* rexp, bool insensitive, bool extended, bool frozen)
+    : JsObject(mtx,name,line,frozen),
       m_regexp(rexp,extended,insensitive)
 {
     XDebug(DebugAll,"JsRegExp::JsRegExp('%s',%p,%s) [%p]",
@@ -1363,17 +1523,17 @@ JsRegExp::JsRegExp(Mutex* mtx, const char* name, const char* rexp, bool insensit
     params().addParam("basicPosix",String::boolText(!extended));
 }
 
-JsRegExp::JsRegExp(Mutex* mtx, const Regexp& rexp, bool frozen)
-    : JsObject(mtx,rexp.c_str()),
+JsRegExp::JsRegExp(ScriptMutex* mtx, unsigned int line, const Regexp& rexp, bool frozen)
+    : JsObject(mtx,rexp.c_str(),line),
       m_regexp(rexp)
 {
     XDebug(DebugAll,"JsRegExp::JsRegExp('%s',%p,%s) [%p]",
 	toString().c_str(),mtx,String::boolText(frozen),this);
 }
 
-JsObject* JsRegExp::copy(Mutex* mtx) const
+JsObject* JsRegExp::copy(ScriptMutex* mtx, const ExpOperation& oper) const
 {
-    JsRegExp* reg = new JsRegExp(mtx,m_regexp,frozen());
+    JsRegExp* reg = new JsRegExp(mtx,oper.lineNumber(),m_regexp,frozen());
     deepCopyParams(reg->params(),params(),mtx);
     return reg;
 }
@@ -1449,7 +1609,7 @@ JsObject* JsRegExp::runConstructor(ObjList& stack, const ExpOperation& oper, Gen
     }
     if (!ref())
 	return 0;
-    JsRegExp* obj = new JsRegExp(mutex(),*pattern,*pattern,insensitive,extended);
+    JsRegExp* obj = new JsRegExp(mutex(),*pattern,oper.lineNumber(),*pattern,insensitive,extended);
     obj->params().addParam(new ExpWrapper(this,protoName()));
     return obj;
 }
@@ -1531,20 +1691,20 @@ JsObject* JsDate::runConstructor(ObjList& stack, const ExpOperation& oper, GenOb
     JsObject* obj = 0;
     switch (extractArgs(stack,oper,context,args)) {
 	case 0:
-	    obj = new JsDate(mutex(),Time::msecNow());
+	    obj = new JsDate(mutex(),oper.lineNumber(),Time::msecNow());
 	    break;
 	case 1:
 	    {
 		ExpOperation* val = static_cast<ExpOperation*>(args[0]);
 		if (val) {
 		    if (val->isInteger())
-			obj = new JsDate(mutex(),val->number());
+			obj = new JsDate(mutex(),oper.lineNumber(),val->number());
 		    else {
 			// Check string
 			uint64_t n = Time::toEpoch(*val,val->length(),1);
 			if (n == (uint64_t)-1)
 			    return JsParser::nullObject();
-			obj = new JsDate(mutex(),n);
+			obj = new JsDate(mutex(),oper.lineNumber(),n);
 		    }
 		}
 	    }
@@ -1572,7 +1732,7 @@ JsObject* JsDate::runConstructor(ObjList& stack, const ExpOperation& oper, GenOb
 		    parts[0] += 1900;
 		parts[1]++;
 		u_int64_t time = Time::toEpoch(parts[0],parts[1],parts[2],parts[3],parts[4],parts[5]);
-		obj = new JsDate(mutex(),1000 * time + parts[6],true);
+		obj = new JsDate(mutex(),oper.lineNumber(),1000 * time + parts[6],true);
 	    }
 	    break;
 	default:
