@@ -28,7 +28,7 @@ namespace TelEngine {
  */
 SDPSession::SDPSession(SDPParser* parser)
     : m_parser(parser), m_mediaStatus(MediaMissing),
-      m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
+      m_rtpForward(false), m_sdpForward(parser->sdpForward()), m_rtpMedia(0),
       m_sdpSession(0), m_sdpVersion(0), m_sdpHash(YSTRING_INIT_HASH),
       m_secure(m_parser->m_secure), m_rfc2833(m_parser->m_rfc2833),
       m_ipv6(false), m_amrExtra(""), m_enabler(0), m_ptr(0)
@@ -38,12 +38,13 @@ SDPSession::SDPSession(SDPParser* parser)
 
 SDPSession::SDPSession(SDPParser* parser, NamedList& params)
     : m_parser(parser), m_mediaStatus(MediaMissing),
-      m_rtpForward(false), m_sdpForward(false), m_rtpMedia(0),
+      m_rtpForward(false), m_sdpForward(parser->sdpForward()), m_rtpMedia(0),
       m_sdpSession(0), m_sdpVersion(0), m_sdpHash(YSTRING_INIT_HASH),
       m_ipv6(false), m_amrExtra(""), m_enabler(0), m_ptr(0)
 {
     setSdpDebug();
     m_rtpForward = params.getBoolValue("rtp_forward");
+    m_sdpForward = params.getBoolValue(YSTRING("forward_sdp"),m_sdpForward);
     m_secure = params.getBoolValue("secure",parser->m_secure);
     m_rfc2833 = parser->m_rfc2833;
     setRfc2833(params.getParam("rfc2833"));
@@ -55,7 +56,7 @@ SDPSession::~SDPSession()
 }
 
 // Set new media list. Return true if changed
-bool SDPSession::setMedia(ObjList* media)
+bool SDPSession::setMedia(ObjList* media, bool preserveExisting)
 {
     if (media == m_rtpMedia)
 	return false;
@@ -67,8 +68,19 @@ bool SDPSession::setMedia(ObjList* media)
 	chg = false;
 	for (ObjList* o = tmp->skipNull(); o; o = o->skipNext()) {
 	    SDPMedia* m = static_cast<SDPMedia*>(o->get());
-	    if (media && m->sameAs(static_cast<SDPMedia*>((*media)[*m]),m_parser->ignorePort()))
-		continue;
+	    if (media) {
+		ObjList* l = media->find(*m);
+		SDPMedia* newMedia = l ? static_cast<SDPMedia*>(l->get()) : 0;
+		if (newMedia && m->sameAs(newMedia,m_parser->ignorePort(),preserveExisting)) {
+		    if (preserveExisting && m->isStarted()) {
+			XDebug(m_enabler,DebugAll,
+			    "SDPSession::setMedia(%p) keeping existing media='%s' format='%s' [%p]",
+			    media,m->c_str(),m->format().c_str(),m_ptr);
+			newMedia->keepRtp(*m);
+		    }
+		    continue;
+		}
+	    }
 	    chg = true;
 	    mediaChanged(*m);
 	}
@@ -179,7 +191,7 @@ bool SDPSession::dispatchRtp(const char* addr, bool start, RefObject* context)
 	    o = o->skipNext();
 	}
 	else {
-	    Debug(m_enabler,DebugMild,
+	    TraceDebug(m_traceId,m_enabler,DebugMild,
 		"Removing failed SDP media '%s' format '%s' from offer [%p]",
 		m->c_str(),m->format().safe(),m_ptr);
 	    o->remove();
@@ -543,11 +555,11 @@ MimeSdpBody* SDPSession::createSDP(const char* addr, ObjList* mediaList)
 
 	if (frm.null()) {
 	    if (m->isAudio() || !m->fmtList()) {
-		Debug(m_enabler,DebugMild,"No formats for '%s', excluding from SDP [%p]",
+		TraceDebug(m_traceId,m_enabler,DebugMild,"No formats for '%s', excluding from SDP [%p]",
 		    m->c_str(),m_ptr);
 		continue;
 	    }
-	    Debug(m_enabler,DebugInfo,"Assuming formats '%s' for media '%s' [%p]",
+	    TraceDebug(m_traceId,m_enabler,DebugInfo,"Assuming formats '%s' for media '%s' [%p]",
 		m->fmtList(),m->c_str(),m_ptr);
 	    frm << " " << m->fmtList();
 	    // brutal but effective
@@ -632,12 +644,9 @@ MimeSdpBody* SDPSession::createPasstroughSDP(NamedList& msg, bool update,
     if (!(m_rtpForward && tmp.toBoolean()))
 	return 0;
     String* raw = msg.getParam("sdp_raw");
-    if (raw) {
-	m_sdpForward = m_sdpForward || m_parser->sdpForward();
-	if (m_sdpForward) {
-	    msg.setParam("rtp_forward","accepted");
-	    return new MimeSdpBody("application/sdp",raw->safe(),raw->length());
-	}
+    if (raw && m_sdpForward) {
+	msg.setParam("rtp_forward","accepted");
+	return new MimeSdpBody("application/sdp",raw->safe(),raw->length());
     }
     String addr;
     ObjList* lst = updateRtpSDP(msg,addr,update ? m_rtpMedia : 0,allowEmptyAddr);
@@ -684,7 +693,7 @@ void SDPSession::updateFormats(const NamedList& msg, bool changeMedia)
 	    SDPMedia* rtp = static_cast<SDPMedia*>(m_rtpMedia->operator[](tmp));
 	    if (!rtp)
 		continue;
-	    Debug(m_enabler,DebugNote,"Removing disabled media '%s' [%p]",
+	    TraceDebug(m_traceId,m_enabler,DebugNote,"Removing disabled media '%s' [%p]",
 		tmp.c_str(),m_ptr);
 	    m_rtpMedia->remove(rtp,false);
 	    mediaChanged(*rtp);
@@ -712,11 +721,11 @@ void SDPSession::updateFormats(const NamedList& msg, bool changeMedia)
 	SDPMedia* rtp = static_cast<SDPMedia*>(m_rtpMedia->operator[](tmp));
 	if (rtp) {
 	    if (rtp->update(*p))
-		Debug(m_enabler,DebugNote,"Formats for '%s' changed to '%s' [%p]",
+		TraceDebug(m_traceId,m_enabler,DebugNote,"Formats for '%s' changed to '%s' [%p]",
 		    tmp.c_str(),rtp->formats().c_str(),m_ptr);
 	}
 	else if (*p) {
-	    Debug(m_enabler,DebugNote,"Got formats '%s' for absent media '%s' [%p]",
+	    TraceDebug(m_traceId,m_enabler,DebugNote,"Got formats '%s' for absent media '%s' [%p]",
 		p->c_str(),tmp.c_str(),m_ptr);
 	    if (trans) {
 		rtp = new SDPMedia(tmp,trans,p->c_str());
@@ -845,22 +854,10 @@ Message* SDPSession::buildChanRtp(SDPMedia* media, const char* addr, bool start,
     m->addParam("remoteip",addr);
     if (start) {
 	m->addParam("remoteport",media->remotePort());
-	String tmp = media->format();
-	tmp << "=";
-	ObjList* mappings = media->mappings().split(',',false);
-	for (ObjList* pl = mappings; pl; pl = pl->next()) {
-	    String* mapping = static_cast<String*>(pl->get());
-	    if (!mapping)
-		continue;
-	    if (mapping->startsWith(tmp)) {
-		tmp = *mapping;
-		tmp >> "=";
-		m->addParam("payload",tmp);
-		break;
-	    }
-	}
+	int payload = SDPMedia::payloadMapping(media->mappings(),media->format());
+	if (payload >= 0)
+	    m->addParam("payload",String(payload));
 	m->addParam("evpayload",media->rfc2833());
-	TelEngine::destruct(mappings);
     }
     if (m_secure) {
 	if (media->remoteCrypto()) {
@@ -875,7 +872,7 @@ Message* SDPSession::buildChanRtp(SDPMedia* media, const char* addr, bool start,
 		    m->addParam("crypto_params",sdes.matchString(4));
 	    }
 	    else
-		Debug(m_enabler,DebugWarn,"Invalid SDES: '%s' [%p]",sdes.c_str(),m_ptr);
+		TraceDebug(m_traceId,m_enabler,DebugWarn,"Invalid SDES: '%s' [%p]",sdes.c_str(),m_ptr);
 	}
 	else if (media->securable())
 	    m->addParam("secure",String::boolText(true));
@@ -1001,10 +998,11 @@ void SDPSession::dispatchingRtp(Message*& msg, SDPMedia* media)
 }
 
 // Set data used in debug
-void SDPSession::setSdpDebug(DebugEnabler* enabler, void* ptr)
+void SDPSession::setSdpDebug(DebugEnabler* enabler, void* ptr, const String& traceId)
 {
     m_enabler = enabler ? enabler : static_cast<DebugEnabler*>(m_parser);
     m_ptr = ptr ? ptr : (void*)this;
+    m_traceId = traceId;
 }
 
 // Print current media to output
@@ -1019,7 +1017,7 @@ void SDPSession::printRtpMedia(const char* reason)
 	    tmp << " ";
 	tmp << m->c_str() << "=" << m->formats();
     }
-    Debug(m_enabler,DebugAll,"%s: %s [%p]",reason,tmp.c_str(),m_ptr);
+    TraceDebug(m_traceId,m_enabler,DebugAll,"%s: %s [%p]",reason,tmp.c_str(),m_ptr);
 }
 
 // Set extra parameters for formats
